<?php
defined( 'ABSPATH' ) || die();
function quoramatic_remove_empty_tags_recursive( $str, $replace_with = null ) {
	//** Return if string not given or empty.
	if ( ! is_string( $str )
	     || trim( $str ) == '' ) {
		return $str;
	}

	//** Recursive empty HTML tags.
	return preg_replace(

	//** Pattern written by Junaid Atari.
		'/<([^<\/>]*)>([\s]*?|(?R))<\/\1>/imsU',

		//** Replace with nothing if string empty.
		! is_string( $replace_with ) ? '' : $replace_with,

		//** Source string
		$str
	);
}
function quoramatic_remove_emoji( $content ) {
	$clean_text = "";

	// Match Emoticons
	$regexEmoticons = '/[\x{1F600}-\x{1F64F}]/u';
	$clean_text     = preg_replace( $regexEmoticons, '', $content );

	// Match Miscellaneous Symbols and Pictographs
	$regexSymbols = '/[\x{1F300}-\x{1F5FF}]/u';
	$clean_text   = preg_replace( $regexSymbols, '', $clean_text );

	// Match Transport And Map Symbols
	$regexTransport = '/[\x{1F680}-\x{1F6FF}]/u';
	$clean_text     = preg_replace( $regexTransport, '', $clean_text );

	// Match Miscellaneous Symbols
	$regexMisc  = '/[\x{2600}-\x{26FF}]/u';
	$clean_text = preg_replace( $regexMisc, '', $clean_text );

	// Match Dingbats
	$regexDingbats = '/[\x{2700}-\x{27BF}]/u';
	$clean_text    = preg_replace( $regexDingbats, '', $clean_text );

	return $clean_text;
}
function quoramatic_remove_unauthorized_html( $content ) {
	$default_allowed_tags = [
		'a'          => array(
			'href'   => true,
			'target' => true,
		),
		'audio'      => array(
			'autoplay' => true,
			'controls' => true,
			'loop'     => true,
			'muted'    => true,
			'preload'  => true,
			'src'      => true,
		),
		'b'          => array(),
		'blockquote' => array(
			'cite'     => true,
			'lang'     => true,
			'xml:lang' => true,
			'class'    => true,
		),
		'br'         => array(),
		'button'     => array(
			'disabled' => true,
			'name'     => true,
			'type'     => true,
			'value'    => true,
		),
		'em'         => array(),
		'h2'         => array(
			'align' => true,
		),
		'h3'         => array(
			'align' => true,
		),
		'h4'         => array(
			'align' => true,
		),
		'h5'         => array(
			'align' => true,
		),
		'h6'         => array(
			'align' => true,
		),
		'i'          => array(),
		'img'        => array(
			'alt'      => true,
			'align'    => true,
			'height'   => true,
			'src'      => true,
			'width'    => true,
			'data-src' => true,
		),
		'p'          => array(
			'xml:lang' => true,
		),
		'table'      => array(),
		'tbody'      => array(),
		'td'         => array(),
		'tfoot'      => array(),
		'th'         => array(),
		'thead'      => array(),
		'tr'         => array(),
		'u'          => array(),
		'ul'         => array(
			'type' => true,
		),
		'ol'         => array(
			'start'    => true,
			'type'     => true,
			'reversed' => true,
		),
		'li'         => array(),
		'iframe'     => array(
			'frameborder' => true,
			'height'      => true,
			'width'       => true,
			'src'         => true,
		)
	];

	$allowed_tags = apply_filters( 'quoramatic_allowed_html_tags', $default_allowed_tags );
	$content      = wp_kses( $content, $allowed_tags );

	return trim( $content );
}
function quoramatic_remove_continue_reading( $content ) {
	$dom = quoramatic_str_get_html( $content );
	/* @var $node simple_html_dom_node */
	foreach ( $dom->find( 'a' ) as $node ) {
		if ( in_array( strtolower( trim( $node->text() ) ), [ 'continue reading', 'read more' ] ) ) {
			$node->remove();
		}
	}

	return $dom;
}
class Quoramatic_Readability_v2 {
	protected $url = '';

	protected $title = '';

	protected $content = '';

	protected $excerpt = '';

	protected $image = '';

	protected $author = '';

	protected $language = '';

	protected $pub_date = '';

	protected $tags = '';

	protected $categories = '';

	protected $meta_description = '';

	/**
	 * @param $url
	 *
	 * @since 1.2.0
	 * Quoramatic_Readability_v2 constructor.
	 */
	public function __construct() {

	}


	/**
	 * @param $html
	 *
	 * @return WP_Error
	 * @since 1.2.0
	 */
	public function parse( $html, $url ) {
		//set properties
		$this->url = esc_url( $url );

		//process full document
		$html = $this->pre_process_html( $html, $this->url );

		$readability                          = new Readability2( $html, $this->url );
		$readability->debug                   = false;
		$readability->convertLinksToFootnotes = false;
		$result                               = $readability->init();

		if ( ! $result ) {
			return new WP_Error( 'readability-error', esc_html__( 'Content is not readable', 'quoramatic-post-generator' ) );
		}

		$title = $readability->getTitle()->textContent;
		$content = quoramatic_remove_empty_tags_recursive( $readability->getContent()->innerHTML );
		$content = quoramatic_remove_emoji( $content );
		$content = force_balance_tags( $content );
		$content = quoramatic_remove_unauthorized_html( $content );
		$content = quoramatic_remove_continue_reading( $content );
		//final content
		$this->content = $content;

		//parse meta
		$metas = $this->parse_meta( $html );

		//title
		if ( array_key_exists( 'og:title', $metas ) ) {
			$title = $metas['og:title'];
		} else if ( array_key_exists( 'twitter:title', $metas ) ) {
			$this->excerpt = $metas['twitter:title'];
		}
		$this->title = $this->clean_title( $title );


		//set image
		if ( array_key_exists( 'og:image', $metas ) ) {
			$this->image = esc_url( $metas['og:image'] );
		} elseif ( array_key_exists( 'twitter:image', $metas ) ) {
			$this->image = esc_url( $metas['twitter:image'] );
		} else {
			$content_dom = quoramatic_str_get_html( $content );
			$img         = $content_dom->find( 'img', 0 );
			if ( $img && $src = $img->getAttribute( 'src' ) ) {
				$this->image = $src;
			}
		}

		//excerpt
		if ( array_key_exists( 'description', $metas ) ) {
			$this->excerpt = $metas['description'];
		} else if ( array_key_exists( 'og:description', $metas ) ) {
			$this->excerpt = $metas['og:description'];
		} else if ( array_key_exists( 'twitter:description', $metas ) ) {
			$this->excerpt = $metas['twitter:description'];
		}

		//language
		if ( array_key_exists( 'og:locale', $metas ) ) {
			$this->language = $metas['og:locale'];
		}

		//author
		//language
		if ( array_key_exists( 'author', $metas ) ) {
			$this->author = $metas['author'];
		}

	}

	/**
	 * @param $html simple_html_dom
	 * @param $url
	 *
	 * @return mixed
	 * @since 1.2.0
	 */
	protected function pre_process_html( $docuemnt, $url ) {
		if ( ! $docuemnt instanceof simple_html_dom ) {
			$docuemnt = quoramatic_str_get_html( $docuemnt );
		}
		//fix image link
		/* @var $img simple_html_dom_node */
		foreach ( $docuemnt->find( 'img' ) as $img ) {
			$urls = [
				$img->getAttribute( 'srcset' ),
				$img->getAttribute( 'src' ),
				$img->getAttribute( 'data-src' ),
				$img->getAttribute( 'data-original' ),
				$img->getAttribute( 'data-orig' ),
				$img->getAttribute( 'data-url' ),
			];

			$src = array_filter( $urls );
			$src = reset( $src );
			$img->setAttribute( 'src', $this->to_absolute_uri( $src, $url ) );
			$img->removeAttribute( 'class' );
			if ( empty( $img->getAttribute( 'src' ) ) ) {
				$img->remove();
			}
		}

		//fix relative url
		foreach ( $docuemnt->find( 'a' ) as $a ) {
			/* @var $a simple_html_dom_node */
			$a->setAttribute( 'href', $this->to_absolute_uri( $a->getAttribute( 'href' ), $url ) );
		}

		return $docuemnt->outertext;
	}

	/**
	 * Parse meta
	 *
	 * @param $html
	 *
	 * @return array
	 * @since 1.2.0
	 */
	protected function parse_meta( $html ) {
		$values = [];
		// Match "description", or Twitter's "twitter:description" (Cards)
		// in name attribute.
		$namePattern = '/^\s*((twitter)\s*:\s*)?(description|title|image|author)\s*$/i';

		// Match Facebook's Open Graph title & description properties.
		$propertyPattern = '/^\s*og\s*:\s*(description|title|image|author)\s*$/i';
		$dom             = quoramatic_str_get_html( $html );

		foreach ( $dom->find( 'meta' ) as $meta ) {
			/* @var $meta simple_html_dom_node */
			$elementName     = $meta->getAttribute( 'name' );
			$elementProperty = $meta->getAttribute( 'property' );

			$name = null;
			if ( preg_match( $namePattern, $elementName ) ) {
				$name = $elementName;
			} elseif ( preg_match( $propertyPattern, $elementProperty ) ) {
				$name = $elementProperty;
			} elseif ( ! empty( $elementProperty ) ) {
				$name = $elementProperty;
			}
			if ( $name ) {
				$content = $meta->getAttribute( 'content' );
				if ( $content ) {
					// Convert to lowercase and remove any whitespace
					// so we can match below.
					$name            = preg_replace( '/\s/', '', strtolower( $name ) );
					$values[ $name ] = trim( $content );
				}
			}
		}

		return $values;
	}

	/**
	 * @param $original_title
	 *
	 * @return string
	 * @since 1.2.0
	 */
	protected function clean_title( $original_title ) {

		if ( $original_title === null ) {
			return null;
		}

		$new_title                      = $original_title;
		$titleHadHierarchicalSeparators = false;

		/*
		 * If there's a separator in the title, first remove the final part
		 *
		 * Sanity warning: if you eval this match in PHPStorm's "Evaluate expression" box, it will return false
		 * I can assure you it works properly if you let the code run.
		 */
		if ( preg_match( '/ [\|\-\\\\\/>»] /i', $new_title ) ) {
			$titleHadHierarchicalSeparators = (bool) preg_match( '/ [\\\\\/>»] /', $new_title );
			$new_title                      = preg_replace( '/(.*)[\|\-\\\\\/>»] .*/i', '$1', $original_title );
			// If the resulting title is too short (3 words or fewer), remove
			// the first part instead:
			if ( count( preg_split( '/\s+/', $new_title ) ) < 3 ) {
				$new_title = preg_replace( '/[^\|\-\\\\\/>»]*[\|\-\\\\\/>»](.*)/i', '$1', $original_title );
			}
		} elseif ( strpos( $new_title, ': ' ) !== false ) {
			if ( count( preg_split( '/\s+/', $new_title ) ) < 3 ) {
				$new_title = substr( $original_title, strpos( $original_title, ':' ) + 1 );
			}
		}

		$new_title = trim( $new_title );

		/*
		 * If we now have 4 words or fewer as our title, and either no
		 * 'hierarchical' separators (\, /, > or ») were found in the original
		 * title or we decreased the number of words by more than 1 word, use
		 * the original title.
		 */
		$new_titleWordCount      = count( preg_split( '/\s+/', $new_title ) );
		$original_titleWordCount = count( preg_split( '/\s+/', preg_replace( '/[\|\-\\\\\/>»]+/', '', $original_title ) ) ) - 1;

		if ( $new_titleWordCount <= 4 &&
		     ( ! $titleHadHierarchicalSeparators || $new_titleWordCount !== $original_titleWordCount ) ) {
			$new_title = $original_title;
		}

		return $new_title;
	}

	/**
	 * Convert link to absolute url
	 *
	 * @param $uri
	 *
	 * @return string
	 * @since 1.2.0
	 */
	protected function to_absolute_uri( $uri, $url ) {
		$scheme   = ! empty( parse_url( $url, PHP_URL_SCHEME ) ) ? parse_url( $url, PHP_URL_SCHEME ) : 'http';
		$pathBase = $scheme . '://' . parse_url( $url, PHP_URL_HOST ) . dirname( parse_url( $url, PHP_URL_PATH ) ) . '/';
		$prePath  = $scheme . '://' . parse_url( $pathBase, PHP_URL_HOST );


		// If this is already an absolute URI, return it.
		if ( preg_match( '/^[a-zA-Z][a-zA-Z0-9\+\-\.]*:/', $uri ) ) {
			return $uri;
		}

		// Scheme-rooted relative URI.
		if ( substr( $uri, 0, 2 ) === '//' ) {
			return $scheme . '://' . substr( $uri, 2 );
		}

		// Prepath-rooted relative URI.
		if ( substr( $uri, 0, 1 ) === '/' ) {
			return trailingslashit( $prePath ) . ltrim( $uri, '/' );
		}

		// Dotslash relative URI.
		if ( strpos( $uri, './' ) === 0 ) {
			return $pathBase . substr( $uri, 2 );
		}
		// Ignore hash URIs:
		if ( substr( $uri, 0, 1 ) === '#' ) {
			return $uri;
		}

		// Standard relative URI; add entire path. pathBase already includes a
		// trailing "/".
		return $pathBase . $uri;
	}


	public function get_title() {
		return $this->title;
	}

	public function get_content() {
		return $this->content;
	}

	public function get_image() {
		return $this->image;
	}

	public function get_excerpt() {
		return $this->excerpt;
	}

	public function get_author() {
		return $this->author;
	}

	public function get_language() {
		return $this->language;
	}

	public function get_pub_date() {
		return $this->pub_date;
	}
}
